/**
 * Copyright (C) 2012 BonitaSoft S.A.
 * BonitaSoft, 32 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.runtime.event;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Vector;

import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;

/**
 * 
 * @author Matthieu Chaffotte
 * 
 */
public class MemoryLockJobExecutor extends JobExecutor {

  private final Object waitForAThreadToBeAvailableSemaphore = new Object();

  private final Vector<ProcessInstanceUUID> locks = new Vector<ProcessInstanceUUID>();

  MemoryLockJobExecutor(final EventExecutor executor, final String name) {
    super(executor, name);
  }

  @Override
  protected Long getNextDueDate() {
    return getCommandService().execute(new GetNextJobDueDate());
  }

  @Override
  public void notifyThreadFinished(final ProcessInstanceUUID rootInstanceUUID) {
    synchronized (waitForAThreadToBeAvailableSemaphore) {
      waitForAThreadToBeAvailableSemaphore.notify();
    }
    locks.remove(rootInstanceUUID);
  }

  @Override
  public void cleanLocks() {
    locks.clear();
  }

  @Override
  protected void execute() throws InterruptedException {
    final Map<ProcessInstanceUUID, List<Long>> jobsOfRootInstances = getCommandService().execute(
        new GetExecutableJobIds((Vector<ProcessInstanceUUID>) locks.clone(), getMaxParallelJobs()));
    for (final Entry<ProcessInstanceUUID, List<Long>> jobsOfRootInstance : jobsOfRootInstances.entrySet()) {
      locks.add(jobsOfRootInstance.getKey());
      final JobExecutorThread thread = new JobExecutorThread(getEventExecutor(), jobsOfRootInstance.getKey(),
          jobsOfRootInstance.getValue());
      getEventExecutor().getThreadPool().submit(thread);
    }

    while (getEventExecutor().getThreadPool().getActiveCount() >= getEventExecutor().getNbrOfThreads()) {
      synchronized (waitForAThreadToBeAvailableSemaphore) {
        waitForAThreadToBeAvailableSemaphore.wait(getEventExecutor().getLockMillis());
      }
    }
  }

  @Override
  protected String getEventMatcherName() {
    return "Job executor";
  }

}
